#!/usr/bin/ruby1.8
#

require "/usr/share/liboml2-dev/oml4r.rb"

APPNAME = "tcpdumpwrap"
APPPATH = "/usr/sbin/tcpdump"
APPVERSION = "1.0"

class MPStat < OML4R::MPBase
    name :tcpdumpmp
    param :hostname
    param :timestamp # timestamp of the received packet
    param :timestamp_us, :type => :long # timestamp of the received packet in us 
    param :rate, :type => :double # data rate of received packet in MB
    param :frequency, :type => :long # frequency in MHz
    param :rssi, :type => :long # RSSI value of the received packet in dBm ?
    param :bssid, :type => :string # bssid
    param :da, :type => :string # destination address
    param :sa, :type => :string # source address
    param :ra, :type => :string # receiving address
    param :frametype, :type => :string # ack , beacon or others
end

class Wrapper
  def process_output(output)
      cmd = "/bin/hostname"
      $node = `#{cmd}`
      $node = $node.delete("\n").strip
      column = output.split(" ")
      if output.include? "signal" then
	timestamp = column[0]
  	timestamp_us = column[1].delete("us")
	rate = column[3]
        frequency = column[5]
	rssi = column[8].delete("dB")
        if output.include? "bssid" then
	  index=testpattern("BSSID",output)
          bssid = column[index].delete("BSSID")
          bssid.slice!(0)
	else
	  bssid="null"
	end
	if output.include?"SA" then
	  index=testpattern("SA",output)
	  sa = column[index].delete("SA")
	  sa.slice!(0)
        else
	  sa="null"
	end
        if output.include?"DA" then
	  index=testpattern("DA",output)
	  da = column[index].delete("DA")
	  da.slice!(0)
	else 
	  da="null"
	end
        if output.include?"RA" then
	  index=testpattern("RA",output)
	  ra = column[index].delete("RA")
	  ra.slice!(0)
	else
	  ra="null"
	end
	if output.include?"Beacon" then
	  frametype="Beacon"
	elsif output.include?"Ack" then
	  frametype="Ack"
	else
	  frametype="Other"
	end
        #puts "output to database #{timestamp},#{timestamp_us},#{rate},#{frequency},#{rssi}, #{bssid},#{da},#{sa},#{ra},#{frametype}"
	MPStat.inject("#{$node}", timestamp, timestamp_us, rate, frequency, rssi, bssid, da, sa, ra, frametype)
      end
  end

  def testpattern(pattern,output)
	column=output.split(" ")
	   valuetoreturn=nil
	   column.each_with_index {|key,index|
	   	if key.include? pattern then
		  valuetoreturn=index
	        end
           }            
        return valuetoreturn
  end


  def initialize(args)
    # Initialise some variable specific to this wrapper
    @interface = nil
    @extensive = false
    @verbose = false
    @protocol = nil

    # Now call the Init of OML4R with the command line arguments (args)
    # and a block defining the arguments specific to this wrapper
    OML4R::init(args, { :appID => APPNAME, :omlServer => '10.11.31.25', })  { |argParser|
        argParser.on("-i interface", "--interface to capture", "Interface") { |name| 
    		@interface = name }
	argParser.on("-e extensive", nil, "extensive") { |ignore|
			@extensive = true }
	argParser.on("-v verbose", nil, "verbose"){ |ignore|
		@verbose = true	}
        argParser.on("-f filter", "--protocolfilter", "protocolfilter"){ |ignore|
		@protocol = ignore }
    }

    # Finally do some checking specific to this wrapper
    # e.g. here we do not proceed if the user did not give us a 
    # valid interface to monitor
    unless @interface != nil
      raise "You did not specify a destination!"
    end
    if @protocol == nil
      debug "!!! The protocol argument is not passed !!! "
    end

  end

  def start()
    cmd = "/sbin/ifconfig #{@interface} up"
    temp = `#{cmd}`
    cmd = ""
	if @extensive && @verbose then
      	cmd = "#{APPPATH} #{@protocol} -i #{@interface} -e -v 2>&1"
	elsif @extensive
      	cmd = "#{APPPATH} #{@protocol} -i #{@interface} -e 2>&1"
	elsif @verbose then
	cmd = "#{APPPATH} #{@protocol} -i #{@interface} -v 2>&1"
	else
	cmd = "#{APPPATH} #{@protocol} -i #{@interface} 2>&1}"
    	end
    output = IO.popen(cmd)
      
    output.each {|line| 
	#print "processing output" + line
        process_output(line)
    }
  end
    


end

begin
  app = Wrapper.new(ARGV)
  app.start()
rescue Exception => ex
  puts "Received an Exception when executing the tcpdumpwrapper!"
  puts "The Exception is: #{ex}\n"
end
